package com.bocloud.ams.booter.config;

import com.bocloud.ams.booter.listener.TaskResultListener;
import com.megatron.common.utils.MapTools;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.RabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {
    private Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");
    @Autowired
    private ConnectionFactory connectionFactory;

    /**
     * 声明审计日志队列
     *
     * @return
     */
    @Bean
    public Queue declareLogQueue() {
        return new Queue("bocloud.log.queue", true, false, false, arguments);
    }


    /**
     * 声明默认的交换
     *
     * @return
     */
    @Bean
    public DirectExchange deafultExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true, arguments);
    }

    /**
     * 绑定日志队列
     *
     * @return
     */
    @Bean
    public Binding bindingLogQueue() {
        return BindingBuilder.bind(declareLogQueue()).to(deafultExchange()).withQueueName();
    }

    /**
     * 绑定CMDB任务队列
     *
     * @return
     */
    @Bean
    public Binding bindingCmdbQueue() {
        return BindingBuilder.bind(declareTaskResultQueue()).to(deafultExchange()).withQueueName();
    }

    /**
     * 声明任务执行队列
     *
     * @return
     */
    @Bean
    public Queue declareTaskResultQueue() {
        return new Queue("cmdb.task.result.queue", true, false, false, arguments);
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        amqpTemplate.setExchange(deafultExchange().getName());
        return amqpTemplate;
    }

    /**
     * 设置手动消费确认
     *
     * @param connectionFactory
     * @return
     */
    @Bean
    public RabbitListenerContainerFactory<?> rabbitListenerContainerFactory(ConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        factory.setConnectionFactory(connectionFactory);
        factory.setMessageConverter(new Jackson2JsonMessageConverter());
        factory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        return factory;
    }

    @Bean
    public SimpleMessageListenerContainer autoJobCompletedListener(TaskResultListener taskResultListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareTaskResultQueue());
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(taskResultListener);
        return container;
    }

    @Bean("customContainerFactory")
    public SimpleRabbitListenerContainerFactory containerFactory(SimpleRabbitListenerContainerFactoryConfigurer configurer, ConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        //设置线程数
        factory.setConcurrentConsumers(10);
        //最大线程数
        factory.setMaxConcurrentConsumers(10);
        configurer.configure(factory, connectionFactory);
        return factory;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitAdmin rabbitAdmin() {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);
        return rabbitAdmin;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public AmqpAdmin amqpAdmin() {
        AmqpAdmin amqpAdmin = new RabbitAdmin(connectionFactory);
        return amqpAdmin;
    }
}
