package com.bocloud.ams.booter.controller.instance;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.model.InstanceBean;
import com.bocloud.ams.entity.model.InstanceTopo;
import com.bocloud.ams.service.instance.InstanceService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/v1/instances")
@Tag(name = "实例管理")
public class InstanceController {

    @Autowired
    private InstanceService instanceService;

    @Operation(tags = {"CMC"}, summary = "实例列表")
    @GetMapping
    public GeneralResult<List<Map<String, Object>>> list(Pager pager,
                                                         @Value(Common.REQ_CONTEXT) RequestContext context,
                                                         @RequestParam(value = "positionKey", required = false) String positionKey) {
        return instanceService.list(pager, context.getTarget(), positionKey);
    }

    /***
     * 新增实例
     *
     * @param instance
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例新增")
    @PostMapping
    public GeneralResult save(@RequestBody JSONObject instance, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceService.create(instance, context.getTarget(), true);
    }

    /***
     * 更新实例
     *
     * @param instance
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例修改")
    @PutMapping(value = "/{id}")
    public GeneralResult modify(@RequestBody JSONObject instance, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceService.modify(instance, context.getTarget());
    }

    /***
     * 批量更新实例
     *
     * @param instance
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例批量更新")
    @PutMapping(value = "/batch")
    public GeneralResult<Void> batchModify(@RequestBody InstanceBean instance,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceService.batchModify(instance, context.getTarget());
    }

    /**
     * 删除实例
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例删除")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @RequestParam(value = "moduleCode") String moduleCode, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceService.remove(id, moduleCode, context.getTarget());
    }

    /**
     * 批量删除实例
     *
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例批量删除")
    @DeleteMapping(value = "/batch")
    public GeneralResult<Void> batchRemove(@RequestBody InstanceBean model,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceService.batchRemove(model, context.getTarget());
    }

    /**
     * 实例查询明细
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例详情")
    @GetMapping(value = "/{id}")
    public GeneralResult<Instance> detail(@PathVariable(value = Common.ID) Long id,
                                          @RequestParam(value = "moduleCode") String moduleCode,
                                          @RequestParam(value = "positionKey", required = false) String positionKey) {
        return instanceService.detail(id, moduleCode, positionKey);
    }

    @Operation(summary = "实例导出")
    @GetMapping(value = "/export")
    public void export(@RequestParam(value = "moduleId") Long moduleId,
                       @RequestParam(value = "moduleCode") String moduleCode,
                       @RequestParam(value = "type") String type,
                       HttpServletRequest request,
                       HttpServletResponse response,
                       @Value(Common.REQ_CONTEXT) RequestContext context,
                       @RequestParam(value = "positionKey", required = false) String positionKey) {
        @SuppressWarnings("unchecked")
        GeneralResult GeneralResult = instanceService.export(moduleId, moduleCode, type, context.getTarget(), positionKey);
        if (GeneralResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result =
                    JSONObject.parseObject(JSONObject.toJSONString(GeneralResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String)result.get("data");

                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                log.error("导出实例数据失败");
            }
        }

    }

    @Operation(summary = "实例模板下载")
    @GetMapping(value = "/download")
    public void download(@RequestParam(value = "moduleId") Long moduleId,
        @RequestParam(value = "moduleCode") String moduleCode, @RequestParam(value = "type") String type,
        HttpServletRequest request, HttpServletResponse response, @Value(Common.REQ_CONTEXT) RequestContext context) {
        @SuppressWarnings("unchecked")
        GeneralResult GeneralResult = instanceService.download(moduleId, moduleCode, type, context.getTarget());
        if (GeneralResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result =
                    JSONObject.parseObject(JSONObject.toJSONString(GeneralResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String)result.get("data");

                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                log.error("下载实例模板失败");
            }
        }
    }

    @Operation(summary = "实例导入")
    @PostMapping(value = "/upload")
    public GeneralResult upload(@RequestParam(value = "moduleId") Long moduleId,
        @RequestParam(value = "moduleCode") String moduleCode,
        @RequestParam(value = "file", required = false) MultipartFile file, HttpServletRequest request,
        HttpServletResponse response, @Value(Common.REQ_CONTEXT) RequestContext context) throws IOException {
        return instanceService.upload(moduleId, moduleCode, file, context);
    }

    @Operation(summary = "实例拓扑")
    @GetMapping(value = "/{id}/topo")
    public GeneralResult<InstanceTopo> topo(@PathVariable(value = Common.ID) Long id,
                                            @RequestParam(value = "relationCategoryId") Long relationCategoryId,
                                            @RequestParam(value = "moduleId") Long moduleId,
                                            @Value(Common.REQ_CONTEXT) RequestContext context,
                                            @RequestParam(value = "positionKey", required = false) String positionKey) {
        return instanceService.topo(id, relationCategoryId, moduleId, context.getTarget(), positionKey);
    }

}
