package com.bocloud.ams.service.instance.impl;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.entity.association.AutoRelationRuleItemValue;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.repository.association.AutoRelationRuleItemRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleItemValueRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleRepository;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.instance.InstanceRelationHistoryRecordRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.bocloud.ams.service.task.InstanceRelationTask;
import com.bocloud.ams.service.task.ModuleInstanceRelationTask;
import com.bocloud.ams.service.utils.RepositoryHandler;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description: 实例自动关联接口实现
 * @Author majiali
 * @CreateTime 2022/6/6 9:10
 */
@Slf4j
@Service("instanceAutoRelationService")
public class InstanceAutoRelationServiceImpl implements InstanceAutoRelationService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private AutoRelationRuleRepository autoRelationRuleRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private AutoRelationRuleItemRepository autoRelationRuleItemRepository;
    @Autowired
    private AutoRelationRuleItemValueRepository autoRelationRuleItemValueRepository;
    @Autowired
    private InstanceRelationHistoryRecordRepository instanceRelationHistoryRecordRepository;
    @Autowired
    private RepositoryHandler handler;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;
    @Autowired
    private OperationalAuditService operationalAuditService;
    @Autowired
    private OperationalAuditRepository operationalAuditRepository;

    @Override
    public GeneralResult<Void> instanceAutoRelation(Instance instance, Long userId) {
        try {
            // 查询实例values
            List<InstanceValue> instanceValues = instanceValueRepository.listByInstanceId(instance.getId(),
                    CmdbUtils.CMDB_INSTANCE_VALUE + instance.getModuleCode().toLowerCase());
            if (CollectionUtils.isEmpty(instanceValues)) {
                // 当前实例没有value，不需要处理关系
                return new GeneralResult<>(false, "实例关系自动关联失败，当前实例没有值数据");
            }
            instance.setInstanceValues(instanceValues);
            // 根据资源数据查询模型关系
            List<ModuleRelation> relations = moduleRelationRepository.queryByModuleId(instance.getModuleId());
            if (CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(false, "实例关系自动关联失败，当前实例模型没有对应的模型关系数据");
            }
            List<AutoRelationRule> rules = autoRelationRuleRepository
                    .queryByModuleRelationIds(relations.stream().map(ModuleRelation::getId).collect(Collectors.toList()));
            if (CollectionUtils.isEmpty(rules)) {
                return new GeneralResult<>(false, "实例关系自动关联失败，当前实例模型关系没有对应的关联规则");
            }
            // 关联规则item及itemvalue相关
            buildAutoRelationRuleData(rules);
            Map<Long, AutoRelationRule> ruleMap =
                    rules.stream().collect(Collectors.toMap(AutoRelationRule::getModuleRelationId, Function.identity(), (oldData, newData) -> newData));
            for (ModuleRelation relation : relations) {
                // 获取模型管理规则
                AutoRelationRule rule = ruleMap.get(relation.getId());
                // 关联规则为空，不需要处理
                if (null == rule || CollectionUtils.isEmpty(rule.getItems())) {
                    continue;
                }
                InstanceRelationTask task = new InstanceRelationTask(rule, relation, instanceRelationRepository,
                        instanceRepository, instanceValueRepository, instanceRelationHistoryRecordRepository, handler,
                        instance, userId, lockFactory,operationalAuditService,operationalAuditRepository);
                threadPoolTaskExecutor.execute(task);
            }
        } catch (Exception e) {
            log.error("实例关系自动关联失败", e);
            return new GeneralResult<>(false, "实例关系自动关联失败");
        }
        return new GeneralResult<>(true, "实例关系自动关联任务已下发");
    }

    @Override
    public GeneralResult<Void> autoRelation() {
        try {
            // 查询所有模型关系
            List<ModuleRelation> relations = moduleRelationRepository.listAll();
            if (CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(false, "实例关系自动关联任务下发失败，模型关系不存在");
            }
            List<AutoRelationRule> rules = autoRelationRuleRepository
                    .queryByModuleRelationIds(relations.stream().map(ModuleRelation::getId).collect(Collectors.toList()));

            if (CollectionUtils.isEmpty(rules)) {
                return new GeneralResult<>(false, "实例关系自动关联任务下发失败，关联规则不存在");
            }
            // 关联规则item及itemvalue相关
            buildAutoRelationRuleData(rules);
            Map<Long, AutoRelationRule> ruleMap =
                    rules.stream().collect(Collectors.toMap(AutoRelationRule::getModuleRelationId, Function.identity(), (oldData, newData) -> newData));
            for (ModuleRelation relation : relations) {
                // 获取模型管理规则
                AutoRelationRule rule = ruleMap.get(relation.getId());
                // 关联规则为空，不需要处理
                if (null == rule || CollectionUtils.isEmpty(rule.getItems())) {
                    continue;
                }
                ModuleInstanceRelationTask task =
                        new ModuleInstanceRelationTask(rule, relation, instanceRelationRepository, instanceRepository,
                                instanceValueRepository, instanceRelationHistoryRecordRepository, handler, lockFactory,operationalAuditService,operationalAuditRepository);
                threadPoolTaskExecutor.execute(task);
            }

        } catch (Exception e) {
            log.error("实例关系自动关联任务下发失败", e);
            return new GeneralResult<>(false, "实例关系自动关联任务下发失败");
        }
        return new GeneralResult<>(true, "实例关系自动关联任务已下发");
    }

    /**
     * @param rules
     * @throws Exception
     * @Description: 查询关联规则对应的item及itemvalue相关
     */
    private void buildAutoRelationRuleData(List<AutoRelationRule> rules) throws Exception {
        for (AutoRelationRule rule : rules) {
            List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(rule.getId());
            for (AutoRelationRuleItem item : items) {
                // 需要查询对应的字段code及类型相关值
                List<AutoRelationRuleItemValue> values =
                        autoRelationRuleItemValueRepository.queryAllByItemId(item.getId());
                item.setValues(values);
            }
            rule.setItems(items);
        }
    }

    @Override
    public GeneralResult<Void> autoRelationByModuleId(Long moduleId) {
        try {
            // 查询模型关系
            List<ModuleRelation> relations = moduleRelationRepository.queryByModuleId(moduleId);
            if (CollectionUtils.isEmpty(relations)) {
                return new GeneralResult<>(false, "实例关系自动关联任务下发失败，模型关系不存在");
            }
            List<AutoRelationRule> rules = autoRelationRuleRepository
                    .queryByModuleRelationIds(relations.stream().map(ModuleRelation::getId).collect(Collectors.toList()));
            if (CollectionUtils.isEmpty(rules)) {
                return new GeneralResult<>(false, "实例关系自动关联任务下发失败，关联规则不存在");
            }
            // 关联规则item及itemvalue相关
            buildAutoRelationRuleData(rules);
            Map<Long, AutoRelationRule> ruleMap =
                    rules.stream().collect(Collectors.toMap(AutoRelationRule::getModuleRelationId, Function.identity(), (oldData, newData) -> newData));
            for (ModuleRelation relation : relations) {
                // 获取模型管理规则
                AutoRelationRule rule = ruleMap.get(relation.getId());
                // 关联规则为空，不需要处理
                if (null == rule || CollectionUtils.isEmpty(rule.getItems())) {
                    continue;
                }
                ModuleInstanceRelationTask task =
                        new ModuleInstanceRelationTask(rule, relation, instanceRelationRepository, instanceRepository,
                                instanceValueRepository, instanceRelationHistoryRecordRepository, handler, lockFactory,operationalAuditService,operationalAuditRepository);
                threadPoolTaskExecutor.execute(task);
            }
        } catch (Exception e) {
            log.error("实例关系自动关联任务下发失败", e);
            return new GeneralResult<>(false, "实例关系自动关联任务下发失败");
        }
        return new GeneralResult<>(true, "实例关系自动关联任务已下发");
    }

}
